<?php

namespace App\Filament\Resources;

use App\Filament\Resources\ServiceTypeResource\Pages;
use App\Filament\Resources\ServiceTypeResource\Widgets\ServiceTypeOverview;
use App\Models\ServiceType;
use Filament\Forms\Components\DatePicker;
use Filament\Forms\Components\Split;
use Filament\Forms\Form;
use Filament\Resources\Resource;
use Filament\Tables;
use Filament\Tables\Table;
use Illuminate\Database\Eloquent\Model;

class ServiceTypeResource extends Resource
{
    private ?string $quantity = 'priv';

    protected static ?string $model = ServiceType::class;

    protected static ?string $navigationIcon = 'heroicon-o-rectangle-stack';

    protected static ?string $navigationGroup = 'Reports';

    protected static ?string $label = 'Product Services';

    public static function getWidgets(): array
    {
        return [
            ServiceTypeOverview::class,
        ];
    }

    public static function form(Form $form): Form
    {
        return $form
            ->schema([
            ]);
    }

    public static function table(Table $table): Table
    {
        return $table
            ->columns([
                Tables\Columns\TextColumn::make('name'),
                Tables\Columns\TextColumn::make('quantity')
                    ->getStateUsing(function (Table $table, Model $record) {
                        return $record->getQuantityAttribute(
                            $table->getFilter('dates')->getState()['created_from'],
                            $table->getFilter('dates')->getState()['created_until']
                        );
                    }),

                Tables\Columns\TextColumn::make('amount')
                    ->getStateUsing(function (Table $table, Model $record) {
                        return $record->getAmountAttribute(
                            $table->getFilter('dates')->getState()['created_from'],
                            $table->getFilter('dates')->getState()['created_until']
                        );
                    })
                    ->prefix('$'),

                Tables\Columns\TextColumn::make('salesPercentage')
                    ->getStateUsing(function (Table $table, Model $record) {
                        return $record->getSalesPercentageAttribute(
                            $table->getFilter('dates')->getState()['created_from'],
                            $table->getFilter('dates')->getState()['created_until']
                        );
                    })
                    ->suffix('%')
                    ->label('Percentage'),

                Tables\Columns\TextColumn::make('averagePrice')
                    ->getStateUsing(function (Table $table, Model $record) {
                        return $record->getAveragePriceAttribute(
                            $table->getFilter('dates')->getState()['created_from'],
                            $table->getFilter('dates')->getState()['created_until']
                        );
                    })
                    ->prefix('$'),
            ])
            ->filters([
                Tables\Filters\Filter::make('dates')
                    ->form([
                        Split::make([
                            DatePicker::make('created_from')
                                ->label('From date'),
                            DatePicker::make('created_until')
                                ->reactive()
                                ->label('Until date'),
                        ]),
                    ]),
            ])
            ->actions([
            ])
            ->bulkActions([
            ]);
    }

    public static function getRelations(): array
    {
        return [
            //
        ];
    }

    public static function getPages(): array
    {
        return [
            'index'  => Pages\ListServiceTypes::route('/'),
            'create' => Pages\CreateServiceType::route('/create'),
            'edit'   => Pages\EditServiceType::route('/{record}/edit'),
        ];
    }
}
