<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Builder;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\Relations\HasMany;

class ServiceType extends Model
{
    use HasFactory;

    protected $fillable = [
        'name',
        'value',
    ];

    protected $appends = [
        'quantity',
        'amount',
        'sales_percentage',
        'average_price',
    ];

    //to do: date between?

    public function getQuantityAttribute($created_at = null, $created_until = null): int
    {
        return $this->productServices()
            ->when($created_at, function (Builder $query) use ($created_at) {
                return $query->whereDate('created_at', '>=', $created_at);
            })
            ->when($created_until, function (Builder $query) use ($created_until) {
                return $query->whereDate('created_at', '<=', $created_until);
            })
            ->sum('amount');
    }

    public function getAmountAttribute($created_at = null, $created_until = null): string
    {
        return number_format(
            $this->productServices()
                ->when($created_at, function (Builder $query) use ($created_at) {
                    return $query->whereDate('created_at', '>=', $created_at);
                })
                ->when($created_until, function (Builder $query) use ($created_until) {
                    return $query->whereDate('created_at', '<=', $created_until);
                })
                ->sum('amount_price'),
            2
        );
    }

    public function getSalesPercentageAttribute($created_at = null, $created_until = null): float
    {
        $total = ProductService::when($created_at, function (Builder $query) use ($created_at) {
            return $query->whereDate('created_at', '>=', $created_at);
        })
            ->when($created_until, function (Builder $query) use ($created_until) {
                return $query->whereDate('created_at', '<=', $created_until);
            })
            ->count();

        $part = ProductService::where('service_type_id', $this->id)
            ->when($created_at, function (Builder $query) use ($created_at) {
                return $query->whereDate('created_at', '>=', $created_at);
            })
            ->when($created_until, function (Builder $query) use ($created_until) {
                return $query->whereDate('created_at', '<=', $created_until);
            })
            ->count();

        return round(($part / $total) * 100, 2);
    }

    public function getAveragePriceAttribute($created_at = null, $created_until = null): string
    {
        return number_format(floatval($this->getAmountAttribute($created_at, $created_until)) /
            $this->getQuantityAttribute($created_at, $created_until), 2);
    }

    public function productServices(): HasMany
    {
        return $this->hasMany(ProductService::class);
    }
}
